@@ -17,7 +17,7 @@ use strict;
 use warnings;
 
 package Clownfish::CFC;
-our $VERSION = '0.005000';
+our $VERSION = '0.005001';
 $VERSION = eval $VERSION;
 our $MAJOR_VERSION = 0.005000;
 
@@ -27,7 +27,7 @@ END {
 }
 
 use XSLoader;
-BEGIN { XSLoader::load( 'Clownfish::CFC', '0.5.0' ) }
+BEGIN { XSLoader::load( 'Clownfish::CFC', '0.5.1' ) }
 
 {
     package Clownfish::CFC::Util;
@@ -268,7 +268,7 @@ BEGIN { XSLoader::load( 'Clownfish::CFC', '0.5.0' ) }
 
             my $class_name = $class->get_name;
             my $rt_class = Clownfish::Class->fetch_class($class_name)
-                or die("Class $class_name not found");
+                or next;
 
             for my $rt_method (@{ $rt_class->get_methods }) {
                 if ($rt_method->is_excluded_from_host) {
