@@ -45,11 +45,6 @@ S_to_c_header_inert(CFCBindClass *self);
 static char*
 S_to_c_header_dynamic(CFCBindClass *self);
 
-// Count the number of member variables declared in ancestor classes
-// outside this package.
-static int
-S_count_non_package_members(CFCBindClass *self);
-
 // Create the definition for the instantiable object struct.
 static char*
 S_struct_definition(CFCBindClass *self);
@@ -70,10 +65,18 @@ S_sub_declarations(CFCBindClass *self);
 static char*
 S_inert_var_declarations(CFCBindClass *self);
 
+// Define type-safe wrappers for Obj functions.
+static char*
+S_wrapper_defs(CFCBindClass *self);
+
 // Define method invocation inline functions.
 static char*
 S_method_defs(CFCBindClass *self);
 
+// Declare override symbols for functions which wrap host callbacks.
+static char*
+S_override_decs(CFCBindClass *self);
+
 // Define short names for all of the symbols associated with this class.
 static char*
 S_short_names(CFCBindClass *self);
@@ -161,7 +164,7 @@ S_ivars_func(CFCBindClass *self) {
     const char *full_offset  = CFCClass_full_ivars_offset(client);
     const char *PREFIX       = CFCClass_get_PREFIX(client);
     char pattern[] =
-        "extern size_t %s;\n"
+        "extern uint32_t %s;\n"
         "typedef struct %s %s;\n"
         "static CFISH_INLINE %s*\n"
         "%s(%s *self) {\n"
@@ -188,15 +191,15 @@ S_ivars_func(CFCBindClass *self) {
 static char*
 S_to_c_header_dynamic(CFCBindClass *self) {
     const char *privacy_symbol  = CFCClass_privacy_symbol(self->client);
-    const char *class_var       = CFCClass_full_class_var(self->client);
-    const char *PREFIX          = CFCClass_get_PREFIX(self->client);
     char *ivars                 = S_ivars_func(self);
     char *struct_def            = S_struct_definition(self);
     char *parent_include        = S_parent_include(self);
     char *sub_declarations      = S_sub_declarations(self);
     char *inert_var_defs        = S_inert_var_declarations(self);
     char *method_typedefs       = S_method_typedefs(self);
+    char *wrapper_defs          = S_wrapper_defs(self);
     char *method_defs           = S_method_defs(self);
+    char *override_decs         = S_override_decs(self);
     char *short_names           = S_short_names(self);
 
     char pattern[] =
@@ -230,15 +233,20 @@ S_to_c_header_dynamic(CFCBindClass *self) {
         "\n"
         "%s\n"
         "\n"
+        "/* Define type-safe wrappers for inert functions of Obj.\n"
+        " */\n"
+        "\n"
+        "%s\n"
+        "\n"
         "/* Define the inline functions which implement this class's virtual methods.\n"
         " */\n"
         "\n"
         "%s\n"
         "\n"
-        "/* Declare the Class singleton for this class.\n"
+        "/* Declare callbacks for wrapping host overrides.\n"
         " */\n"
         "\n"
-        "extern %sVISIBLE cfish_Class *%s;\n"
+        "%s\n"
         "\n"
         "/* Define \"short names\" for this class's symbols.\n"
         " */\n"
@@ -248,8 +256,8 @@ S_to_c_header_dynamic(CFCBindClass *self) {
     char *content
         = CFCUtil_sprintf(pattern, parent_include, privacy_symbol, ivars,
                           struct_def, privacy_symbol, inert_var_defs,
-                          sub_declarations, method_typedefs, method_defs,
-                          PREFIX, class_var, short_names);
+                          sub_declarations, method_typedefs, wrapper_defs,
+                          method_defs, override_decs, short_names);
 
     FREEMEM(ivars);
     FREEMEM(struct_def);
@@ -257,7 +265,9 @@ S_to_c_header_dynamic(CFCBindClass *self) {
     FREEMEM(sub_declarations);
     FREEMEM(inert_var_defs);
     FREEMEM(method_typedefs);
+    FREEMEM(wrapper_defs);
     FREEMEM(method_defs);
+    FREEMEM(override_decs);
     FREEMEM(short_names);
     return content;
 }
@@ -265,105 +275,36 @@ S_to_c_header_dynamic(CFCBindClass *self) {
 char*
 CFCBindClass_to_c_data(CFCBindClass *self) {
     CFCClass *client = self->client;
-    const char *class_name = CFCClass_get_class_name(client);
 
     if (CFCClass_inert(client)) {
         return CFCUtil_strdup("");
     }
 
     const char *ivars_offset = CFCClass_full_ivars_offset(client);
-
     const char *class_var    = CFCClass_full_class_var(client);
 
     CFCMethod **methods  = CFCClass_methods(client);
 
     char *offsets           = CFCUtil_strdup("");
     char *method_defs       = CFCUtil_strdup("");
-    char *novel_ms_var      = CFCUtil_strdup("");
-    char *overridden_ms_var = CFCUtil_strdup("");
-    char *inherited_ms_var  = CFCUtil_strdup("");
 
     for (int meth_num = 0; methods[meth_num] != NULL; meth_num++) {
         CFCMethod *method = methods[meth_num];
 
         // Define method offset variable.
         char *full_offset_sym = CFCMethod_full_offset_sym(method, client);
-        offsets = CFCUtil_cat(offsets, "size_t ", full_offset_sym, ";\n",
+        offsets = CFCUtil_cat(offsets, "uint32_t ", full_offset_sym, ";\n",
                               NULL);
         FREEMEM(full_offset_sym);
 
-        const char *meth_class_name = CFCMethod_get_class_name(method);
-        int is_fresh = strcmp(class_name, meth_class_name) == 0;
+        int is_fresh = CFCMethod_is_fresh(method, client);
 
         // Create a default implementation for abstract methods.
         if (is_fresh && CFCMethod_abstract(method)) {
-            char *method_def = CFCBindMeth_abstract_method_def(method);
+            char *method_def = CFCBindMeth_abstract_method_def(method, client);
             method_defs = CFCUtil_cat(method_defs, method_def, "\n", NULL);
             FREEMEM(method_def);
         }
-
-        if (is_fresh && CFCMethod_novel(method)) {
-            if (novel_ms_var[0] == '\0') {
-                // Start an array of cfish_NovelMethSpec structs.  Since C89
-                // doesn't allow us to initialize a pointer to an anonymous
-                // array inside a global struct, we have to give it a real
-                // symbol and then store a pointer to that symbol inside the
-                // ClassSpec struct.
-                novel_ms_var
-                    = CFCUtil_cat(novel_ms_var,
-                                  "static const cfish_NovelMethSpec ",
-                                  class_var, "_NOVEL_METHS[] = {\n", NULL);
-            }
-            else {
-                novel_ms_var = CFCUtil_cat(novel_ms_var, ",\n", NULL);
-            }
-            char *ms_def = CFCBindMeth_novel_spec_def(method);
-            novel_ms_var = CFCUtil_cat(novel_ms_var, ms_def, NULL);
-            FREEMEM(ms_def);
-        }
-        else if (is_fresh) {
-            if (overridden_ms_var[0] == '\0') {
-                // Start an array of cfish_OverriddenMethSpec structs.
-                overridden_ms_var
-                    = CFCUtil_cat(overridden_ms_var,
-                                  "static const cfish_OverriddenMethSpec ",
-                                  class_var, "_OVERRIDDEN_METHS[] = {\n",
-                                  NULL);
-            }
-            else {
-                overridden_ms_var
-                    = CFCUtil_cat(overridden_ms_var, ",\n", NULL);
-            }
-            char *ms_def = CFCBindMeth_overridden_spec_def(method, client);
-            overridden_ms_var = CFCUtil_cat(overridden_ms_var, ms_def, NULL);
-            FREEMEM(ms_def);
-        }
-        else {
-            if (inherited_ms_var[0] == '\0') {
-                // Start an array of cfish_InheritedMethSpec structs.
-                inherited_ms_var
-                    = CFCUtil_cat(inherited_ms_var,
-                                  "static const cfish_InheritedMethSpec ",
-                                  class_var, "_INHERITED_METHS[] = {\n", NULL);
-            }
-            else {
-                inherited_ms_var = CFCUtil_cat(inherited_ms_var, ",\n", NULL);
-            }
-            char *ms_def = CFCBindMeth_inherited_spec_def(method, client);
-            inherited_ms_var = CFCUtil_cat(inherited_ms_var, ms_def, NULL);
-            FREEMEM(ms_def);
-        }
-    }
-
-    // Close MethSpec array definitions.
-    if (novel_ms_var[0] != '\0') {
-        novel_ms_var = CFCUtil_cat(novel_ms_var, "\n};\n\n", NULL);
-    }
-    if (overridden_ms_var[0] != '\0') {
-        overridden_ms_var = CFCUtil_cat(overridden_ms_var, "\n};\n\n", NULL);
-    }
-    if (inherited_ms_var[0] != '\0') {
-        inherited_ms_var = CFCUtil_cat(inherited_ms_var, "\n};\n\n", NULL);
     }
 
     const char pattern[] =
@@ -371,7 +312,7 @@ CFCBindClass_to_c_data(CFCBindClass *self) {
         " * can be found.\n"
         " */\n"
         "\n"
-        "size_t %s;\n"
+        "uint32_t %s;\n"
         "\n"
         "/* Offsets for method pointers, measured in bytes, from the top\n"
         " * of this class's singleton object.\n"
@@ -384,12 +325,6 @@ CFCBindClass_to_c_data(CFCBindClass *self) {
         "\n"
         "%s\n"
         "\n"
-        "/* Define the MethSpec structs used during Class initialization.\n"
-        " */\n"
-        "\n"
-        "%s"
-        "%s"
-        "%s"
         "/* Define the pointer to the Class singleton object.\n"
         " */\n"
         "\n"
@@ -397,36 +332,13 @@ CFCBindClass_to_c_data(CFCBindClass *self) {
         "\n";
     char *code
         = CFCUtil_sprintf(pattern, ivars_offset, offsets, method_defs,
-                          novel_ms_var, overridden_ms_var, inherited_ms_var,
                           class_var);
 
     FREEMEM(offsets);
     FREEMEM(method_defs);
-    FREEMEM(novel_ms_var);
-    FREEMEM(overridden_ms_var);
-    FREEMEM(inherited_ms_var);
     return code;
 }
 
-// Count the number of member variables declared in ancestor classes
-// outside this package.
-static int
-S_count_non_package_members(CFCBindClass *self) {
-    CFCClass  *const client = self->client;
-    CFCParcel *parcel       = CFCClass_get_parcel(client);
-    CFCClass  *ancestor     = CFCClass_get_parent(client);
-    int num_non_package_members = 0;
-
-    while (ancestor && CFCClass_get_parcel(ancestor) == parcel) {
-        ancestor = CFCClass_get_parent(ancestor);
-    }
-    if (ancestor) {
-        num_non_package_members = CFCClass_num_member_vars(ancestor);
-    }
-
-    return num_non_package_members;
-}
-
 // Create the definition for the instantiable object struct.
 static char*
 S_struct_definition(CFCBindClass *self) {
@@ -434,8 +346,8 @@ S_struct_definition(CFCBindClass *self) {
     const char *struct_sym;
     char       *member_decs = CFCUtil_strdup("");
 
-    const char *prefix = CFCClass_get_prefix(client);
-    if (strcmp(prefix, "cfish_") == 0) {
+    CFCParcel *parcel = CFCClass_get_parcel(client);
+    if (CFCParcel_is_cfish(parcel)) {
         struct_sym = CFCClass_full_struct_sym(client);
         member_decs = CFCUtil_cat(member_decs, "\n    CFISH_OBJ_HEAD", NULL);
     }
@@ -445,8 +357,8 @@ S_struct_definition(CFCBindClass *self) {
 
     // Add all member variables declared by classes in this package.
     CFCVariable **member_vars = CFCClass_member_vars(client);
-    int num_non_package_members = S_count_non_package_members(self);
-    for (int i = num_non_package_members; member_vars[i] != NULL; i++) {
+    size_t num_non_package_members = CFCClass_num_non_package_ivars(client);
+    for (size_t i = num_non_package_members; member_vars[i] != NULL; i++) {
         const char *member_dec = CFCVariable_local_declaration(member_vars[i]);
         member_decs = CFCUtil_cat(member_decs, "\n    ", member_dec, NULL);
     }
@@ -466,132 +378,6 @@ S_struct_definition(CFCBindClass *self) {
     return struct_def;
 }
 
-// Return C definition of the class's ClassSpec.
-char*
-CFCBindClass_spec_def(CFCBindClass *self) {
-    CFCClass *client = self->client;
-
-    CFCClass   *parent       = CFCClass_get_parent(client);
-    const char *class_name   = CFCClass_get_class_name(client);
-    const char *class_var    = CFCClass_full_class_var(client);
-    const char *struct_sym   = CFCClass_full_struct_sym(client);
-    const char *ivars_struct = CFCClass_full_ivars_struct(client);
-    const char *prefix       = CFCClass_get_prefix(client);
-
-    // Create a pointer to the parent Class object.
-    char *parent_ref;
-    if (parent) {
-        parent_ref = CFCUtil_sprintf("&%s", CFCClass_full_class_var(parent));
-    }
-    else {
-        // No parent, e.g. Obj or inert classes.
-        parent_ref = CFCUtil_strdup("NULL");
-    }
-
-    int num_novel      = 0;
-    int num_overridden = 0;
-    int num_inherited  = 0;
-    CFCMethod **methods = CFCClass_methods(client);
-
-    for (int meth_num = 0; methods[meth_num] != NULL; meth_num++) {
-        CFCMethod *method = methods[meth_num];
-        const char *meth_class_name = CFCMethod_get_class_name(method);
-
-        if (strcmp(class_name, meth_class_name) == 0) {
-            if (CFCMethod_novel(method)) {
-                ++num_novel;
-            }
-            else {
-                ++num_overridden;
-            }
-        }
-        else {
-            ++num_inherited;
-        }
-    }
-
-    char *novel_ms_var      = num_novel
-                              ? CFCUtil_sprintf("%s_NOVEL_METHS", class_var)
-                              : CFCUtil_strdup("NULL");
-    char *overridden_ms_var = num_overridden
-                              ? CFCUtil_sprintf("%s_OVERRIDDEN_METHS",
-                                                class_var)
-                              : CFCUtil_strdup("NULL");
-    char *inherited_ms_var  = num_inherited
-                              ? CFCUtil_sprintf("%s_INHERITED_METHS",
-                                                class_var)
-                              : CFCUtil_strdup("NULL");
-
-    char *ivars_size = NULL;
-
-    if (strcmp(prefix, "cfish_") == 0) {
-        ivars_size = CFCUtil_sprintf("sizeof(%s)", struct_sym);
-    }
-    else {
-        int num_non_package_members = S_count_non_package_members(self);
-        int num_members             = CFCClass_num_member_vars(client);
-
-        if (num_non_package_members == num_members) {
-            // No members in this package.
-            ivars_size = CFCUtil_strdup("0");
-        }
-        else {
-            ivars_size = CFCUtil_sprintf("sizeof(%s)", ivars_struct);
-        }
-    }
-    const char *ivars_offset_name = CFCClass_full_ivars_offset(client);
-
-    char pattern[] =
-        "    {\n"
-        "        &%s, /* class */\n"
-        "        %s, /* parent */\n"
-        "        \"%s\", /* name */\n"
-        "        %s, /* ivars_size */\n"
-        "        &%s, /* ivars_offset_ptr */\n"
-        "        %d, /* num_novel */\n"
-        "        %d, /* num_overridden */\n"
-        "        %d, /* num_inherited */\n"
-        "        %s, /* novel_meth_specs */\n"
-        "        %s, /* overridden_meth_specs */\n"
-        "        %s /* inherited_meth_specs */\n"
-        "    }";
-    char *code
-        = CFCUtil_sprintf(pattern, class_var, parent_ref, class_name,
-                          ivars_size, ivars_offset_name, num_novel,
-                          num_overridden, num_inherited, novel_ms_var,
-                          overridden_ms_var, inherited_ms_var);
-
-    FREEMEM(parent_ref);
-    FREEMEM(novel_ms_var);
-    FREEMEM(overridden_ms_var);
-    FREEMEM(inherited_ms_var);
-    FREEMEM(ivars_size);
-    return code;
-}
-
-// Declare host callbacks.
-char*
-CFCBindClass_callback_decs(CFCBindClass *self) {
-    CFCClass   *client        = self->client;
-    CFCMethod **fresh_methods = CFCClass_fresh_methods(client);
-    char       *cb_decs       = CFCUtil_strdup("");
-
-    for (int meth_num = 0; fresh_methods[meth_num] != NULL; meth_num++) {
-        CFCMethod *method = fresh_methods[meth_num];
-
-        // Declare callback.
-        if (CFCMethod_novel(method) && !CFCMethod_final(method)) {
-            char *cb_dec = CFCBindMeth_callback_dec(method);
-            cb_decs = CFCUtil_cat(cb_decs, cb_dec, "\n", NULL);
-            FREEMEM(cb_dec);
-        }
-    }
-
-    FREEMEM(fresh_methods);
-
-    return cb_decs;
-}
-
 // Declare typedefs for every method, to ease casting.
 static char*
 S_method_typedefs(CFCBindClass *self) {
@@ -627,7 +413,7 @@ S_sub_declarations(CFCBindClass *self) {
     char *declarations = CFCUtil_strdup("");
     for (int i = 0; functions[i] != NULL; i++) {
         CFCFunction *func = functions[i];
-        char *dec = CFCBindFunc_func_declaration(func);
+        char *dec = CFCBindFunc_func_declaration(func, self->client);
         if (!CFCFunction_inline(func)) {
             declarations = CFCUtil_cat(declarations, PREFIX, "VISIBLE ", NULL);
         }
@@ -636,14 +422,10 @@ S_sub_declarations(CFCBindClass *self) {
     }
     for (int i = 0; fresh_methods[i] != NULL; i++) {
         CFCMethod *method = fresh_methods[i];
-        char *dec = CFCBindMeth_imp_declaration(method);
-        if (CFCMethod_final(method)) {
-            declarations = CFCUtil_cat(declarations, PREFIX, "VISIBLE ", NULL);
-        }
+        char *dec = CFCBindMeth_imp_declaration(method, self->client);
         declarations = CFCUtil_cat(declarations, dec, "\n\n", NULL);
         FREEMEM(dec);
     }
-    FREEMEM(fresh_methods);
     return declarations;
 }
 
@@ -654,13 +436,49 @@ S_inert_var_declarations(CFCBindClass *self) {
     CFCVariable **inert_vars = CFCClass_inert_vars(self->client);
     char *declarations = CFCUtil_strdup("");
     for (int i = 0; inert_vars[i] != NULL; i++) {
-        const char *global_c = CFCVariable_global_c(inert_vars[i]);
+        char *global_c = CFCVariable_global_c(inert_vars[i], self->client);
         declarations = CFCUtil_cat(declarations, "extern ", PREFIX, "VISIBLE ",
                                    global_c, ";\n", NULL);
+        FREEMEM(global_c);
     }
     return declarations;
 }
 
+// Define type-safe wrappers for Obj functions.
+static char*
+S_wrapper_defs(CFCBindClass *self) {
+    CFCClass *client = self->client;
+
+    if (strcmp(CFCClass_get_name(client), "Clownfish::Obj") == 0) {
+        return CFCUtil_strdup("");
+    }
+
+    const char *prefix     = CFCClass_get_prefix(client);
+    const char *nickname   = CFCClass_get_nickname(client);
+    const char *struct_sym = CFCClass_full_struct_sym(client);
+
+    const char *pattern =
+        "static CFISH_INLINE cfish_Class*\n"
+        "%s%s_get_class(%s *self) {\n"
+        "    return cfish_Obj_get_class((cfish_Obj*)self);\n"
+        "}\n"
+        "\n"
+        "static CFISH_INLINE cfish_String*\n"
+        "%s%s_get_class_name(%s *self) {\n"
+        "    return cfish_Obj_get_class_name((cfish_Obj*)self);\n"
+        "}\n"
+        "\n"
+        "static CFISH_INLINE bool\n"
+        "%s%s_is_a(%s *self, cfish_Class *ancestor) {\n"
+        "    return cfish_Obj_is_a((cfish_Obj*)self, ancestor);\n"
+        "}\n";
+
+    return CFCUtil_sprintf(pattern,
+                           prefix, nickname, struct_sym,
+                           prefix, nickname, struct_sym,
+                           prefix, nickname, struct_sym);
+}
+
 // Define method invocation inline functions.
 static char*
 S_method_defs(CFCBindClass *self) {
@@ -675,6 +493,47 @@ S_method_defs(CFCBindClass *self) {
     return method_defs;
 }
 
+static char*
+S_override_decs(CFCBindClass *self) {
+    CFCMethod **fresh_methods = CFCClass_fresh_methods(self->client);
+    char *decs  = CFCUtil_strdup("");
+    char *nulls = CFCUtil_strdup("");
+
+    for (int i = 0; fresh_methods[i] != NULL; i++) {
+        CFCMethod *method = fresh_methods[i];
+        if (CFCMethod_final(method) || !CFCMethod_novel(method)) {
+            continue;
+        }
+        char *override_sym = CFCMethod_full_override_sym(method, self->client);
+        CFCType      *return_type  = CFCMethod_get_return_type(method);
+        CFCParamList *param_list   = CFCMethod_get_param_list(method);
+        const char   *ret_type_str = CFCType_to_c(return_type);
+        const char   *params       = CFCParamList_to_c(param_list);
+        char pattern[] =
+            "%s\n"
+            "%s(%s);\n";
+        char *callback_dec
+            = CFCUtil_sprintf(pattern, ret_type_str, override_sym, params);
+        decs = CFCUtil_cat(decs, callback_dec, NULL);
+        FREEMEM(callback_dec);
+
+        nulls = CFCUtil_cat(nulls, "#define ", override_sym, " NULL\n", NULL);
+        FREEMEM(override_sym);
+    }
+
+    char pattern[] =
+        "#ifdef CFISH_NO_DYNAMIC_OVERRIDES\n"
+        "%s"
+        "#else\n"
+        "%s"
+        "#endif\n"
+        ;
+    char *content = CFCUtil_sprintf(pattern, nulls, decs);
+
+    FREEMEM(nulls);
+    FREEMEM(decs);
+    return content;
+}
 
 // Define short names for all of the symbols associated with this class.
 static char*
@@ -698,18 +557,44 @@ S_short_names(CFCBindClass *self) {
     CFCFunction **functions = CFCClass_functions(client);
     for (int i = 0; functions[i] != NULL; i++) {
         CFCFunction *func = functions[i];
-        short_names = CFCUtil_cat(short_names, "  #define ",
-                                  CFCFunction_short_func_sym(func), " ",
-                                  CFCFunction_full_func_sym(func), "\n",
-                                  NULL);
+        char *short_sym = CFCFunction_short_func_sym(func, client);
+        char *full_sym  = CFCFunction_full_func_sym(func, client);
+        short_names = CFCUtil_cat(short_names, "  #define ", short_sym, " ",
+                                  full_sym, "\n", NULL);
+        FREEMEM(short_sym);
+        FREEMEM(full_sym);
     }
 
     CFCVariable **inert_vars = CFCClass_inert_vars(client);
     for (int i = 0; inert_vars[i] != NULL; i++) {
         CFCVariable *var = inert_vars[i];
-        short_names = CFCUtil_cat(short_names, "  #define ",
-                                  CFCVariable_short_sym(var), " ",
-                                  CFCVariable_full_sym(var), "\n", NULL);
+        char *short_sym = CFCVariable_short_sym(var, client);
+        char *full_sym  = CFCVariable_full_sym(var, client);
+        short_names = CFCUtil_cat(short_names, "  #define ", short_sym, " ",
+                                  full_sym, "\n", NULL);
+        FREEMEM(short_sym);
+        FREEMEM(full_sym);
+    }
+
+    // Wrappers.
+    if (!CFCClass_inert(client)
+        && strcmp(CFCClass_get_name(client), "Clownfish::Obj") != 0
+       ) {
+        static const char *wrapped_funcs[] = {
+            "get_class",
+            "get_class_name",
+            "is_a"
+        };
+        static int num_wrapped_funcs
+            = sizeof(wrapped_funcs) / sizeof(wrapped_funcs[0]);
+        const char *prefix   = CFCClass_get_prefix(client);
+        const char *nickname = CFCClass_get_nickname(client);
+        for (int i = 0; i < num_wrapped_funcs; i++) {
+            const char *func = wrapped_funcs[i];
+            short_names
+                = CFCUtil_cat(short_names, "  #define ", nickname, "_", func,
+                              " ", prefix, nickname, "_", func, "\n", NULL);
+        }
     }
 
     if (!CFCClass_inert(client)) {
@@ -718,12 +603,13 @@ S_short_names(CFCBindClass *self) {
             CFCMethod *meth = fresh_methods[i];
 
             // Implementing functions.
-            const char *short_imp  = CFCMethod_short_imp_func(meth);
-            const char *full_imp   = CFCMethod_imp_func(meth);
+            char *short_imp = CFCMethod_short_imp_func(meth, client);
+            char *full_imp  = CFCMethod_imp_func(meth, client);
             short_names = CFCUtil_cat(short_names, "  #define ", short_imp,
                                       " ", full_imp, "\n", NULL);
+            FREEMEM(short_imp);
+            FREEMEM(full_imp);
         }
-        FREEMEM(fresh_methods);
 
         CFCMethod  **methods = CFCClass_methods(client);
         for (int i = 0; methods[i] != NULL; i++) {
