@@ -318,12 +318,14 @@ CFCCHtml_write_html_docs(CFCCHtml *self) {
         char *html_doc = html_docs[i];
         CFCUtil_write_if_changed(path, html_doc, strlen(html_doc));
         FREEMEM(html_doc);
+        FREEMEM(dir);
         FREEMEM(path);
         FREEMEM(filename);
     }
 
     FREEMEM(html_docs);
     FREEMEM(filenames);
+    FREEMEM(md_docs);
     FREEMEM(ordered);
 }
 
@@ -475,7 +477,7 @@ S_create_standalone_doc(CFCCHtml *self, CFCDocument *doc) {
     char *title  = CFCUtil_global_replace(path, CHY_DIR_SEP, "::");
     char *header = CFCUtil_global_replace(self->header, "{title}", title);
 
-    const char *md = CFCDocument_get_contents(doc);
+    char *md = CFCDocument_get_contents(doc);
     int dir_level = 0;
     for (size_t i = 0; path[i]; i++) {
         if (path[i] == CHY_DIR_SEP_CHAR) { ++dir_level; }
@@ -485,6 +487,7 @@ S_create_standalone_doc(CFCCHtml *self, CFCDocument *doc) {
     char *html_doc = CFCUtil_sprintf("%s%s%s", header, body, self->footer);
 
     FREEMEM(body);
+    FREEMEM(md);
     FREEMEM(header);
     FREEMEM(title);
     return html_doc;
@@ -946,11 +949,10 @@ S_html_create_inheritance(CFCClass *klass) {
 static char*
 S_md_to_html(const char *md, CFCClass *klass, int dir_level) {
     int options = CMARK_OPT_SMART
-                  | CMARK_OPT_VALIDATE_UTF8
-                  | CMARK_OPT_SAFE;
+                  | CMARK_OPT_VALIDATE_UTF8;
     cmark_node *doc = cmark_parse_document(md, strlen(md), options);
     S_transform_doc(doc, klass, dir_level);
-    char *html = cmark_render_html(doc, CMARK_OPT_DEFAULT);
+    char *html = cmark_render_html(doc, CMARK_OPT_SAFE);
     cmark_node_free(doc);
 
     return html;
