@@ -23,9 +23,10 @@
   #define false 0
 #endif
 
-#define CFC_NEED_SYMBOL_STRUCT_DEF
-#include "CFCSymbol.h"
+#define CFC_NEED_BASE_STRUCT_DEF
+#include "CFCBase.h"
 #include "CFCClass.h"
+#include "CFCSymbol.h"
 #include "CFCFunction.h"
 #include "CFCMethod.h"
 #include "CFCParcel.h"
@@ -48,7 +49,11 @@ static void
 S_register(CFCClass *self);
 
 struct CFCClass {
-    CFCSymbol symbol;
+    CFCBase base;
+    struct CFCParcel *parcel;
+    char *exposure;
+    char *name;
+    char *nickname;
     int tree_grown;
     CFCDocuComment *docucomment;
     struct CFCClass *parent;
@@ -56,8 +61,12 @@ struct CFCClass {
     size_t num_kids;
     CFCFunction **functions;
     size_t num_functions;
+    CFCMethod **fresh_methods;
+    size_t num_fresh_meths;
     CFCMethod **methods;
     size_t num_methods;
+    CFCVariable **fresh_vars;
+    size_t num_fresh_vars;
     CFCVariable **member_vars;
     size_t num_member_vars;
     CFCVariable **inert_vars;
@@ -66,6 +75,7 @@ struct CFCClass {
     char *parent_class_name;
     int is_final;
     int is_inert;
+    int is_abstract;
     char *struct_sym;
     char *full_struct_sym;
     char *ivars_struct;
@@ -99,42 +109,144 @@ static const CFCMeta CFCCLASS_META = {
 
 CFCClass*
 CFCClass_create(struct CFCParcel *parcel, const char *exposure,
-                const char *class_name, const char *nickname,
-                const char *micro_sym, CFCDocuComment *docucomment,
-                CFCFileSpec *file_spec, const char *parent_class_name,
-                int is_final, int is_inert) {
+                const char *name, const char *nickname,
+                CFCDocuComment *docucomment, CFCFileSpec *file_spec,
+                const char *parent_class_name, int is_final, int is_inert,
+                int is_abstract) {
     CFCClass *self = (CFCClass*)CFCBase_allocate(&CFCCLASS_META);
-    return CFCClass_do_create(self, parcel, exposure, class_name, nickname,
-                              micro_sym, docucomment, file_spec,
-                              parent_class_name, is_final, is_inert);
+    return CFCClass_do_create(self, parcel, exposure, name, nickname,
+                              docucomment, file_spec, parent_class_name,
+                              is_final, is_inert, is_abstract);
+}
+
+static int
+S_validate_exposure(const char *exposure) {
+    if (!exposure) { return false; }
+    if (strcmp(exposure, "public")
+        && strcmp(exposure, "parcel")
+        && strcmp(exposure, "private")
+       ) {
+        return false;
+    }
+    return true;
+}
+
+int
+CFCClass_validate_class_name(const char *class_name) {
+    // The last component must contain lowercase letters (for now).
+    const char *last_colon = strrchr(class_name, ':');
+    const char *substring = last_colon ? last_colon + 1 : class_name;
+    for (;;substring++) {
+        if (*substring == 0)          { return false; }
+        else if (*substring == ':')   { return false; }
+        else if (islower(*substring)) { break; }
+    }
+
+    // Must be UpperCamelCase, separated by "::".
+    const char *ptr = class_name;
+    if (!isupper(*ptr)) { return false; }
+    while (*ptr != 0) {
+        if (*ptr == 0) { break; }
+        else if (*ptr == ':') {
+            ptr++;
+            if (*ptr != ':') { return false; }
+            ptr++;
+            if (!isupper(*ptr)) { return false; }
+            ptr++;
+        }
+        else if (!isalnum(*ptr)) { return false; }
+        else { ptr++; }
+    }
+
+    return true;
+}
+
+int
+CFCClass_validate_class_name_component(const char *name) {
+    if (!name || !strlen(name)) { return false; }
+    if (!CFCClass_validate_class_name(name)) { return false; }
+    if (strchr(name, ':') != NULL) { return false; }
+    return true;
+}
+
+static int
+S_validate_nickname(const char *nickname) {
+    // Allow all caps.
+    const char *ptr;
+    for (ptr = nickname; ; ptr++) {
+        if (*ptr == 0) {
+            if (strlen(nickname)) { return true; }
+            else { break; }
+        }
+        else if (!isupper(*ptr)) { break; }
+    }
+
+    // Same as one component of a class name.
+    return CFCClass_validate_class_name_component(nickname);
 }
 
 CFCClass*
 CFCClass_do_create(CFCClass *self, struct CFCParcel *parcel,
-                   const char *exposure, const char *class_name,
-                   const char *nickname, const char *micro_sym,
-                   CFCDocuComment *docucomment, CFCFileSpec *file_spec,
-                   const char *parent_class_name, int is_final, int is_inert) {
-    CFCUTIL_NULL_CHECK(class_name);
-    exposure  = exposure  ? exposure  : "parcel";
-    micro_sym = micro_sym ? micro_sym : "class";
-    CFCSymbol_init((CFCSymbol*)self, parcel, exposure, class_name, nickname,
-                   micro_sym);
+                   const char *exposure, const char *name,
+                   const char *nickname, CFCDocuComment *docucomment,
+                   CFCFileSpec *file_spec, const char *parent_class_name,
+                   int is_final, int is_inert, int is_abstract) {
+    CFCUTIL_NULL_CHECK(parcel);
+    CFCUTIL_NULL_CHECK(name);
+    exposure = exposure ? exposure  : "parcel";
+
+    // Validate.
+    if (!S_validate_exposure(exposure)) {
+        CFCBase_decref((CFCBase*)self);
+        CFCUtil_die("Invalid exposure: '%s'", exposure);
+    }
+    if (!CFCClass_validate_class_name(name)) {
+        CFCBase_decref((CFCBase*)self);
+        CFCUtil_die("Invalid name: '%s'", name);
+    }
+
+    const char *last_colon = strrchr(name, ':');
+    const char *struct_sym = last_colon ? last_colon + 1 : name;
+
+    // Derive nickname if necessary, then validate.
+    const char *real_nickname = NULL;
+    if (nickname) {
+        real_nickname = nickname;
+    }
+    else {
+        real_nickname = struct_sym;
+    }
+    if (!S_validate_nickname(real_nickname)) {
+        CFCBase_decref((CFCBase*)self);
+        CFCUtil_die("Invalid nickname: '%s'", real_nickname);
+    }
+
+    // Default parent class name is "Clownfish::Obj".
     if (!is_inert
         && !parent_class_name
-        && strcmp(class_name, "Clownfish::Obj") != 0
+        && strcmp(name, "Clownfish::Obj") != 0
        ) {
         parent_class_name = "Clownfish::Obj";
     }
-    self->parent     = NULL;
-    self->tree_grown = false;
+
+    // Assign.
+    self->parcel          = (CFCParcel*)CFCBase_incref((CFCBase*)parcel);
+    self->exposure        = CFCUtil_strdup(exposure);
+    self->name            = CFCUtil_strdup(name);
+    self->nickname        = CFCUtil_strdup(real_nickname);
+    self->tree_grown      = false;
+    self->parent          = NULL;
     self->children        = (CFCClass**)CALLOCATE(1, sizeof(CFCClass*));
     self->num_kids        = 0;
     self->functions       = (CFCFunction**)CALLOCATE(1, sizeof(CFCFunction*));
     self->num_functions   = 0;
-    self->methods         = (CFCMethod**)CALLOCATE(1, sizeof(CFCMethod*));
+    self->fresh_methods   = (CFCMethod**)CALLOCATE(1, sizeof(CFCMethod*));
+    self->num_fresh_meths = 0;
+    self->methods         = NULL;
     self->num_methods     = 0;
-    self->member_vars     = (CFCVariable**)CALLOCATE(1, sizeof(CFCVariable*));
+    self->fresh_vars      = (CFCVariable**)CALLOCATE(1, sizeof(CFCVariable*));
+    self->num_fresh_vars  = 0;
+    self->member_vars     = NULL;
     self->num_member_vars = 0;
     self->inert_vars      = (CFCVariable**)CALLOCATE(1, sizeof(CFCVariable*));
     self->num_inert_vars  = 0;
@@ -144,33 +256,30 @@ CFCClass_do_create(CFCClass *self, struct CFCParcel *parcel,
     self->file_spec = (CFCFileSpec*)CFCBase_incref((CFCBase*)file_spec);
 
     // Cache several derived symbols.
-    const char *last_colon = strrchr(class_name, ':');
-    self->struct_sym = last_colon
-                       ? CFCUtil_strdup(last_colon + 1)
-                       : CFCUtil_strdup(class_name);
+
     const char *prefix = CFCClass_get_prefix(self);
-    size_t struct_sym_len = strlen(self->struct_sym);
-    self->short_class_var = (char*)MALLOCATE(struct_sym_len + 1);
+    self->struct_sym        = CFCUtil_strdup(struct_sym);
+    self->full_struct_sym   = CFCUtil_sprintf("%s%s", prefix, struct_sym);
+    self->ivars_struct      = CFCUtil_sprintf("%sIVARS", struct_sym);
+    self->full_ivars_struct = CFCUtil_sprintf("%s%s", prefix,
+                                              self->ivars_struct);
+    self->ivars_func        = CFCUtil_sprintf("%s_IVARS", self->nickname);
+    self->full_ivars_func   = CFCUtil_sprintf("%s%s", prefix,
+                                              self->ivars_func);
+    self->full_ivars_offset = CFCUtil_sprintf("%s_OFFSET",
+                                              self->full_ivars_func);
+
+    const char *PREFIX = CFCClass_get_PREFIX(self);
+    size_t struct_sym_len = strlen(struct_sym);
+    char *short_class_var = (char*)MALLOCATE(struct_sym_len + 1);
     size_t i;
     for (i = 0; i < struct_sym_len; i++) {
-        self->short_class_var[i] = toupper(self->struct_sym[i]);
+        short_class_var[i] = toupper(struct_sym[i]);
     }
-    self->short_class_var[struct_sym_len] = '\0';
-    self->full_struct_sym   = CFCUtil_sprintf("%s%s", prefix, self->struct_sym);
-    self->ivars_struct      = CFCUtil_sprintf("%sIVARS", self->struct_sym);
-    self->full_ivars_struct = CFCUtil_sprintf("%sIVARS", self->full_struct_sym);
-    self->ivars_func        = CFCUtil_sprintf("%s_IVARS",
-                                              CFCClass_get_nickname(self));
-    self->full_ivars_func   = CFCUtil_sprintf("%s%s_IVARS", prefix,
-                                              CFCClass_get_nickname(self));
-    self->full_ivars_offset = CFCUtil_sprintf("%s_OFFSET", self->full_ivars_func);
-    size_t full_struct_sym_len = strlen(self->full_struct_sym);
-    self->full_class_var = (char*)MALLOCATE(full_struct_sym_len + 1);
-    for (i = 0; self->full_struct_sym[i] != '\0'; i++) {
-        self->full_class_var[i] = toupper(self->full_struct_sym[i]);
-    }
-    self->full_class_var[i] = '\0';
-    self->privacy_symbol = CFCUtil_sprintf("C_%s", self->full_class_var);
+    short_class_var[struct_sym_len] = '\0';
+    self->short_class_var = short_class_var;
+    self->full_class_var  = CFCUtil_sprintf("%s%s", PREFIX, short_class_var);
+    self->privacy_symbol  = CFCUtil_sprintf("C_%s", self->full_class_var);
 
     // Build the relative path to the autogenerated C header file.
     if (file_spec) {
@@ -183,11 +292,13 @@ CFCClass_do_create(CFCClass *self, struct CFCParcel *parcel,
 
     self->is_final    = !!is_final;
     self->is_inert    = !!is_inert;
+    self->is_abstract = !!is_abstract;
 
+    // Check for include flag mismatch.
     if (!CFCClass_included(self) && CFCParcel_included(parcel)) {
         CFCUtil_die("Class %s from source dir found in parcel %s from"
                     " include dir",
-                    class_name, CFCParcel_get_name(parcel));
+                    name, CFCParcel_get_name(parcel));
     }
 
     // Skip class if it's from an include dir and the parcel was already
@@ -208,31 +319,32 @@ CFCClass_do_create(CFCClass *self, struct CFCParcel *parcel,
     return self;
 }
 
+static void
+S_free_cfcbase_array(CFCBase **array) {
+    if (array != NULL) {
+        for (size_t i = 0; array[i] != NULL; i++) {
+            CFCBase_decref(array[i]);
+        }
+        FREEMEM(array);
+    }
+}
+
 void
 CFCClass_destroy(CFCClass *self) {
+    CFCBase_decref((CFCBase*)self->parcel);
+    FREEMEM(self->exposure);
+    FREEMEM(self->name);
+    FREEMEM(self->nickname);
     CFCBase_decref((CFCBase*)self->docucomment);
     CFCBase_decref((CFCBase*)self->parent);
-    for (size_t i = 0; self->children[i] != NULL; i++) {
-        CFCBase_decref((CFCBase*)self->children[i]);
-    }
-    for (size_t i = 0; self->functions[i] != NULL; i++) {
-        CFCBase_decref((CFCBase*)self->functions[i]);
-    }
-    for (size_t i = 0; self->methods[i] != NULL; i++) {
-        CFCBase_decref((CFCBase*)self->methods[i]);
-    }
-    for (size_t i = 0; self->member_vars[i] != NULL; i++) {
-        CFCBase_decref((CFCBase*)self->member_vars[i]);
-    }
-    for (size_t i = 0; self->inert_vars[i] != NULL; i++) {
-        CFCBase_decref((CFCBase*)self->inert_vars[i]);
-    }
     CFCBase_decref((CFCBase*)self->file_spec);
-    FREEMEM(self->children);
-    FREEMEM(self->functions);
-    FREEMEM(self->methods);
-    FREEMEM(self->member_vars);
-    FREEMEM(self->inert_vars);
+    S_free_cfcbase_array((CFCBase**)self->children);
+    S_free_cfcbase_array((CFCBase**)self->functions);
+    S_free_cfcbase_array((CFCBase**)self->fresh_methods);
+    S_free_cfcbase_array((CFCBase**)self->methods);
+    S_free_cfcbase_array((CFCBase**)self->fresh_vars);
+    S_free_cfcbase_array((CFCBase**)self->member_vars);
+    S_free_cfcbase_array((CFCBase**)self->inert_vars);
     FREEMEM(self->parent_class_name);
     FREEMEM(self->struct_sym);
     FREEMEM(self->ivars_struct);
@@ -245,7 +357,7 @@ CFCClass_destroy(CFCClass *self) {
     FREEMEM(self->full_class_var);
     FREEMEM(self->privacy_symbol);
     FREEMEM(self->include_h);
-    CFCSymbol_destroy((CFCSymbol*)self);
+    CFCBase_destroy((CFCBase*)self);
 }
 
 static void
@@ -262,31 +374,29 @@ S_register(CFCClass *self) {
         registry_cap = new_cap;
     }
 
-    CFCParcel  *parcel     = CFCClass_get_parcel(self);
-    const char *prefix     = CFCParcel_get_prefix(parcel);
-    const char *class_name = CFCClass_get_class_name(self);
-    const char *nickname   = CFCClass_get_nickname(self);
-    const char *key        = self->full_struct_sym;
+    const char *prefix   = CFCParcel_get_prefix(self->parcel);
+    const char *name     = self->name;
+    const char *nickname = self->nickname;
+    const char *key      = self->full_struct_sym;
 
     for (size_t i = 0; i < registry_size; i++) {
-        CFCClass   *other            = registry[i].klass;
-        CFCParcel  *other_parcel     = CFCClass_get_parcel(other);
-        const char *other_prefix     = CFCParcel_get_prefix(other_parcel);
-        const char *other_class_name = CFCClass_get_class_name(other);
-        const char *other_nickname   = CFCClass_get_nickname(other);
+        CFCClass   *other          = registry[i].klass;
+        const char *other_prefix   = CFCParcel_get_prefix(other->parcel);
+        const char *other_name     = other->name;
+        const char *other_nickname = other->nickname;
 
-        if (strcmp(class_name, other_class_name) == 0) {
-            CFCUtil_die("Two classes with name %s", class_name);
+        if (strcmp(name, other_name) == 0) {
+            CFCUtil_die("Two classes with name %s", name);
         }
         if (strcmp(registry[i].key, key) == 0) {
             CFCUtil_die("Class name conflict between %s and %s",
-                        class_name, other_class_name);
+                        name, other_name);
         }
         if (strcmp(prefix, other_prefix) == 0
             && strcmp(nickname, other_nickname) == 0
            ) {
             CFCUtil_die("Class nickname conflict between %s and %s",
-                        class_name, other_class_name);
+                        name, other_name);
         }
     }
 
@@ -314,6 +424,14 @@ CFCClass_fetch_singleton(CFCParcel *parcel, const char *class_name) {
     }
     char key[MAX_SINGLETON_LEN + 1];
     sprintf(key, "%s%s", prefix, struct_sym);
+
+    return CFCClass_fetch_by_struct_sym(key);
+}
+
+CFCClass*
+CFCClass_fetch_by_struct_sym(const char *key) {
+    CFCUTIL_NULL_CHECK(key);
+
     for (size_t i = 0; i < registry_size; i++) {
         if (strcmp(registry[i].key, key) == 0) {
             return registry[i].klass;
@@ -347,12 +465,10 @@ CFCClass_add_child(CFCClass *self, CFCClass *child) {
         CFCUtil_die("Can't call add_child after grow_tree");
     }
     if (self->is_inert) {
-        CFCUtil_die("Can't inherit from inert class %s",
-                    CFCClass_get_class_name(self));
+        CFCUtil_die("Can't inherit from inert class %s", self->name);
     }
     if (child->is_inert) {
-        CFCUtil_die("Inert class %s can't inherit",
-                    CFCClass_get_class_name(child));
+        CFCUtil_die("Inert class %s can't inherit", child->name);
     }
     self->num_kids++;
     size_t size = (self->num_kids + 1) * sizeof(CFCClass*);
@@ -367,8 +483,7 @@ CFCClass_add_child(CFCClass *self, CFCClass *child) {
     if (!CFCParcel_has_prereq(child_parcel, parcel)) {
         CFCUtil_die("Class '%s' inherits from '%s', but parcel '%s' is not a"
                     " prerequisite of '%s'",
-                    CFCClass_get_class_name(child),
-                    CFCClass_get_class_name(self),
+                    child->name, self->name,
                     CFCParcel_get_name(parcel),
                     CFCParcel_get_name(child_parcel));
     }
@@ -398,12 +513,12 @@ CFCClass_add_method(CFCClass *self, CFCMethod *method) {
     if (self->is_inert) {
         CFCUtil_die("Can't add_method to an inert class");
     }
-    self->num_methods++;
-    size_t size = (self->num_methods + 1) * sizeof(CFCMethod*);
-    self->methods = (CFCMethod**)REALLOCATE(self->methods, size);
-    self->methods[self->num_methods - 1]
+    self->num_fresh_meths++;
+    size_t size = (self->num_fresh_meths + 1) * sizeof(CFCMethod*);
+    self->fresh_methods = (CFCMethod**)REALLOCATE(self->fresh_methods, size);
+    self->fresh_methods[self->num_fresh_meths - 1]
         = (CFCMethod*)CFCBase_incref((CFCBase*)method);
-    self->methods[self->num_methods] = NULL;
+    self->fresh_methods[self->num_fresh_meths] = NULL;
 }
 
 void
@@ -412,12 +527,12 @@ CFCClass_add_member_var(CFCClass *self, CFCVariable *var) {
     if (self->tree_grown) {
         CFCUtil_die("Can't call add_member_var after grow_tree");
     }
-    self->num_member_vars++;
-    size_t size = (self->num_member_vars + 1) * sizeof(CFCVariable*);
-    self->member_vars = (CFCVariable**)REALLOCATE(self->member_vars, size);
-    self->member_vars[self->num_member_vars - 1]
+    self->num_fresh_vars++;
+    size_t size = (self->num_fresh_vars + 1) * sizeof(CFCVariable*);
+    self->fresh_vars = (CFCVariable**)REALLOCATE(self->fresh_vars, size);
+    self->fresh_vars[self->num_fresh_vars - 1]
         = (CFCVariable*)CFCBase_incref((CFCBase*)var);
-    self->member_vars[self->num_member_vars] = NULL;
+    self->fresh_vars[self->num_fresh_vars] = NULL;
 }
 
 void
@@ -442,15 +557,9 @@ S_find_func(CFCFunction **funcs, const char *sym) {
         return NULL;
     }
 
-    char lcsym[MAX_FUNC_LEN + 1];
-    size_t sym_len = strlen(sym);
-    if (sym_len > MAX_FUNC_LEN) { CFCUtil_die("sym too long: '%s'", sym); }
-    for (size_t i = 0; i <= sym_len; i++) {
-        lcsym[i] = tolower(sym[i]);
-    }
     for (size_t i = 0; funcs[i] != NULL; i++) {
         CFCFunction *func = funcs[i];
-        if (strcmp(lcsym, CFCFunction_micro_sym(func)) == 0) {
+        if (strcmp(sym, CFCFunction_get_name(func)) == 0) {
             return func;
         }
     }
@@ -469,15 +578,7 @@ CFCClass_method(CFCClass *self, const char *sym) {
 
 CFCMethod*
 CFCClass_fresh_method(CFCClass *self, const char *sym) {
-    CFCMethod *method = CFCClass_method(self, sym);
-    if (method) {
-        const char *class_name = CFCClass_get_class_name(self);
-        const char *meth_class_name = CFCMethod_get_class_name(method);
-        if (strcmp(class_name, meth_class_name) == 0) {
-            return method;
-        }
-    }
-    return NULL;
+    return (CFCMethod*)S_find_func((CFCFunction**)self->fresh_methods, sym);
 }
 
 void
@@ -485,11 +586,11 @@ CFCClass_resolve_types(CFCClass *self) {
     for (size_t i = 0; self->functions[i] != NULL; i++) {
         CFCFunction_resolve_types(self->functions[i]);
     }
-    for (size_t i = 0; self->methods[i] != NULL; i++) {
-        CFCMethod_resolve_types(self->methods[i]);
+    for (size_t i = 0; self->fresh_methods[i] != NULL; i++) {
+        CFCMethod_resolve_types(self->fresh_methods[i]);
     }
-    for (size_t i = 0; self->member_vars[i] != NULL; i++) {
-        CFCVariable_resolve_type(self->member_vars[i]);
+    for (size_t i = 0; self->fresh_vars[i] != NULL; i++) {
+        CFCVariable_resolve_type(self->fresh_vars[i]);
     }
     for (size_t i = 0; self->inert_vars[i] != NULL; i++) {
         CFCVariable_resolve_type(self->inert_vars[i]);
@@ -501,16 +602,16 @@ static void
 S_bequeath_member_vars(CFCClass *self) {
     for (size_t i = 0; self->children[i] != NULL; i++) {
         CFCClass *child = self->children[i];
-        size_t num_vars = self->num_member_vars + child->num_member_vars;
+        size_t num_vars = self->num_member_vars + child->num_fresh_vars;
         size_t size = (num_vars + 1) * sizeof(CFCVariable*);
         child->member_vars
             = (CFCVariable**)REALLOCATE(child->member_vars, size);
-        memmove(child->member_vars + self->num_member_vars,
-                child->member_vars,
-                child->num_member_vars * sizeof(CFCVariable*));
         memcpy(child->member_vars, self->member_vars,
                self->num_member_vars * sizeof(CFCVariable*));
-        for (size_t j = 0; self->member_vars[j] != NULL; j++) {
+        memcpy(child->member_vars + self->num_member_vars,
+               child->fresh_vars,
+               child->num_fresh_vars * sizeof(CFCVariable*));
+        for (size_t j = 0; j < num_vars; j++) {
             CFCBase_incref((CFCBase*)child->member_vars[j]);
         }
         child->num_member_vars = num_vars;
@@ -526,15 +627,15 @@ S_bequeath_methods(CFCClass *self) {
 
         // Create array of methods, preserving exact order so vtables match up.
         size_t num_methods = 0;
-        size_t max_methods = self->num_methods + child->num_methods;
+        size_t max_methods = self->num_methods + child->num_fresh_meths;
         CFCMethod **methods = (CFCMethod**)MALLOCATE(
                                   (max_methods + 1) * sizeof(CFCMethod*));
 
         // Gather methods which child inherits or overrides.
         for (size_t i = 0; i < self->num_methods; i++) {
             CFCMethod *method = self->methods[i];
-            const char *macro_sym = CFCMethod_get_macro_sym(method);
-            CFCMethod *child_method = CFCClass_method(child, macro_sym);
+            const char *meth_name = CFCMethod_get_name(method);
+            CFCMethod *child_method = CFCClass_fresh_method(child, meth_name);
             if (child_method) {
                 CFCMethod_override(child_method, method);
                 methods[num_methods++] = child_method;
@@ -546,8 +647,8 @@ S_bequeath_methods(CFCClass *self) {
 
         // Append novel child methods to array.  Child methods which were just
         // marked via CFCMethod_override() a moment ago are skipped.
-        for (size_t i = 0; i < child->num_methods; i++) {
-            CFCMethod *method = child->methods[i];
+        for (size_t i = 0; i < child->num_fresh_meths; i++) {
+            CFCMethod *method = child->fresh_methods[i];
             if (CFCMethod_novel(method)) {
                 methods[num_methods++] = method;
             }
@@ -571,10 +672,6 @@ S_bequeath_methods(CFCClass *self) {
                 CFCBase_incref((CFCBase*)methods[i]);
             }
         }
-        for (size_t i = 0; i < child->num_methods; i++) {
-            CFCBase_decref((CFCBase*)child->methods[i]);
-        }
-        FREEMEM(child->methods);
         child->methods     = methods;
         child->num_methods = num_methods;
 
@@ -605,14 +702,39 @@ S_family_tree_size(CFCClass *self) {
     return count;
 }
 
+static CFCBase**
+S_copy_cfcbase_array(CFCBase **array, size_t num_elems) {
+    CFCBase **copy = (CFCBase**)MALLOCATE((num_elems + 1) * sizeof(CFCBase*));
+    for (size_t i = 0; i < num_elems; i++) {
+        copy[i] = CFCBase_incref(array[i]);
+    }
+    copy[num_elems] = NULL;
+    return copy;
+}
+
 void
 CFCClass_grow_tree(CFCClass *self) {
     if (self->tree_grown) {
         CFCUtil_die("Can't call grow_tree more than once");
     }
     S_establish_ancestry(self);
+
+    // Copy fresh variabless for root class.
+    self->member_vars
+        = (CFCVariable**)S_copy_cfcbase_array((CFCBase**)self->fresh_vars,
+                                              self->num_fresh_vars);
+    self->num_member_vars = self->num_fresh_vars;
+
     S_bequeath_member_vars(self);
+
+    // Copy fresh methods for root class.
+    self->methods
+        = (CFCMethod**)S_copy_cfcbase_array((CFCBase**)self->fresh_methods,
+                                            self->num_fresh_meths);
+    self->num_methods = self->num_fresh_meths;
+
     S_bequeath_methods(self);
+
     self->tree_grown = 1;
 }
 
@@ -636,33 +758,14 @@ CFCClass_tree_to_ladder(CFCClass *self) {
     return ladder;
 }
 
-static CFCSymbol**
-S_fresh_syms(CFCClass *self, CFCSymbol **syms) {
-    const char *class_name = CFCClass_get_class_name(self);
-    size_t count = 0;
-    while (syms[count] != NULL) { count++; }
-    size_t amount = (count + 1) * sizeof(CFCSymbol*);
-    CFCSymbol **fresh = (CFCSymbol**)MALLOCATE(amount);
-    size_t num_fresh = 0;
-    for (size_t i = 0; i < count; i++) {
-        CFCSymbol *sym = syms[i];
-        const char *sym_class_name = CFCSymbol_get_class_name(sym);
-        if (strcmp(sym_class_name, class_name) == 0) {
-            fresh[num_fresh++] = sym;
-        }
-    }
-    fresh[num_fresh] = NULL;
-    return fresh;
-}
-
 CFCMethod**
 CFCClass_fresh_methods(CFCClass *self) {
-    return (CFCMethod**)S_fresh_syms(self, (CFCSymbol**)self->methods);
+    return self->fresh_methods;
 }
 
 CFCVariable**
 CFCClass_fresh_member_vars(CFCClass *self) {
-    return (CFCVariable**)S_fresh_syms(self, (CFCSymbol**)self->member_vars);
+    return self->fresh_vars;
 }
 
 CFCClass**
@@ -677,24 +780,54 @@ CFCClass_functions(CFCClass *self) {
 
 CFCMethod**
 CFCClass_methods(CFCClass *self) {
+    if (!self->tree_grown) {
+        CFCUtil_die("Can't call 'methods' before 'grow_tree'");
+    }
     return self->methods;
 }
 
 size_t
 CFCClass_num_methods(CFCClass *self) {
+    if (!self->tree_grown) {
+        CFCUtil_die("Can't call 'num_methods' before 'grow_tree'");
+    }
     return self->num_methods;
 }
 
 CFCVariable**
 CFCClass_member_vars(CFCClass *self) {
+    if (!self->tree_grown) {
+        CFCUtil_die("Can't call 'member_vars' before 'grow_tree'");
+    }
     return self->member_vars;
 }
 
 size_t
 CFCClass_num_member_vars(CFCClass *self) {
+    if (!self->tree_grown) {
+        CFCUtil_die("Can't call 'num_member_vars' before 'grow_tree'");
+    }
     return self->num_member_vars;
 }
 
+// Count the number of member variables declared in ancestor classes
+// outside this package.
+size_t
+CFCClass_num_non_package_ivars(CFCClass *self) {
+    CFCParcel *parcel       = CFCClass_get_parcel(self);
+    CFCClass  *ancestor     = CFCClass_get_parent(self);
+    size_t num_non_package_members = 0;
+
+    while (ancestor && CFCClass_get_parcel(ancestor) == parcel) {
+        ancestor = CFCClass_get_parent(ancestor);
+    }
+    if (ancestor) {
+        num_non_package_members = CFCClass_num_member_vars(ancestor);
+    }
+
+    return num_non_package_members;
+}
+
 CFCVariable**
 CFCClass_inert_vars(CFCClass *self) {
     return self->inert_vars;
@@ -702,7 +835,7 @@ CFCClass_inert_vars(CFCClass *self) {
 
 const char*
 CFCClass_get_nickname(CFCClass *self) {
-    return CFCSymbol_get_class_nickname((CFCSymbol*)self);
+    return self->nickname;
 }
 
 void
@@ -749,6 +882,11 @@ CFCClass_inert(CFCClass *self) {
     return self->is_inert;
 }
 
+int
+CFCClass_abstract(CFCClass *self) {
+    return self->is_abstract;
+}
+
 const char*
 CFCClass_get_struct_sym(CFCClass *self) {
     return self->struct_sym;
@@ -804,33 +942,43 @@ CFCClass_include_h(CFCClass *self) {
     return self->include_h;
 }
 
+CFCParcel*
+CFCClass_get_parcel(CFCClass *self) {
+    return self->parcel;
+}
+
+const char*
+CFCClass_get_prefix(CFCClass *self) {
+    return CFCParcel_get_prefix(self->parcel);
+}
+
+const char*
+CFCClass_get_Prefix(CFCClass *self) {
+    return CFCParcel_get_Prefix(self->parcel);
+}
+
+const char*
+CFCClass_get_PREFIX(CFCClass *self) {
+    return CFCParcel_get_PREFIX(self->parcel);
+}
+
+const char*
+CFCClass_get_exposure(CFCClass *self) {
+    return self->exposure;
+}
+
+int
+CFCClass_public(CFCClass *self) {
+    return !strcmp(self->exposure, "public");
+}
+
+const char*
+CFCClass_get_name(CFCClass *self) {
+    return self->name;
+}
+
 struct CFCDocuComment*
 CFCClass_get_docucomment(CFCClass *self) {
     return self->docucomment;
 }
 
-const char*
-CFCClass_get_prefix(CFCClass *self) {
-    return CFCSymbol_get_prefix((CFCSymbol*)self);
-}
-
-const char*
-CFCClass_get_Prefix(CFCClass *self) {
-    return CFCSymbol_get_Prefix((CFCSymbol*)self);
-}
-
-const char*
-CFCClass_get_PREFIX(CFCClass *self) {
-    return CFCSymbol_get_PREFIX((CFCSymbol*)self);
-}
-
-const char*
-CFCClass_get_class_name(CFCClass *self) {
-    return CFCSymbol_get_class_name((CFCSymbol*)self);
-}
-
-CFCParcel*
-CFCClass_get_parcel(CFCClass *self) {
-    return CFCSymbol_get_parcel((CFCSymbol*)self);
-}
-
