@@ -38,6 +38,7 @@
 #include "CFCSymbol.h"
 #include "CFCUtil.h"
 #include "CFCParser.h"
+#include "CFCDocument.h"
 
 struct CFCHierarchy {
     CFCBase base;
@@ -79,10 +80,13 @@ static void
 S_parse_cf_files(CFCHierarchy *self, const char *source_dir, int is_included);
 
 static void
-S_find_files(const char *path, void *arg);
+S_find_doc_files(const char *source_dir);
 
 static void
-S_free_find_files_context(CFCFindFilesContext *context);
+S_find_files(const char *path, void *arg);
+
+static char*
+S_extract_path_part(const char *path, const char *dir, const char *ext);
 
 static void
 S_connect_classes(CFCHierarchy *self);
@@ -153,21 +157,12 @@ CFCHierarchy_destroy(CFCHierarchy *self) {
     for (size_t i = 0; self->classes[i] != NULL; i++) {
         CFCBase_decref((CFCBase*)self->classes[i]);
     }
-    for (size_t i = 0; self->sources[i] != NULL; i++) {
-        FREEMEM(self->sources[i]);
-    }
-    for (size_t i = 0; self->includes[i] != NULL; i++) {
-        FREEMEM(self->includes[i]);
-    }
-    for (size_t i = 0; self->prereqs[i] != NULL; i++) {
-        FREEMEM(self->prereqs[i]);
-    }
+    CFCUtil_free_string_array(self->sources);
+    CFCUtil_free_string_array(self->includes);
+    CFCUtil_free_string_array(self->prereqs);
     FREEMEM(self->trees);
     FREEMEM(self->files);
     FREEMEM(self->classes);
-    FREEMEM(self->sources);
-    FREEMEM(self->includes);
-    FREEMEM(self->prereqs);
     FREEMEM(self->dest);
     FREEMEM(self->inc_dest);
     FREEMEM(self->src_dest);
@@ -237,9 +232,10 @@ CFCHierarchy_build(CFCHierarchy *self) {
 
     S_check_prereqs(self);
 
-    // Read .cfh files.
+    // Read .cfh and .md files.
     for (size_t i = 0; self->sources[i] != NULL; i++) {
         S_parse_cf_files(self, self->sources[i], false);
+        S_find_doc_files(self->sources[i]);
     }
     for (size_t i = 0; self->includes[i] != NULL; i++) {
         S_parse_cf_files(self, self->includes[i], true);
@@ -263,26 +259,10 @@ S_parse_parcel_files(const char *source_dir, int is_included) {
     context.num_paths = 0;
     CFCUtil_walk(source_dir, S_find_files, &context);
 
-    size_t source_dir_len = strlen(source_dir);
-
     // Parse .cfp files and register the parcels they define.
     for (int i = 0; context.paths[i] != NULL; i++) {
         const char *path = context.paths[i];
-
-        if (strncmp(path, source_dir, source_dir_len) != 0) {
-            CFCUtil_die("'%s' doesn't start with '%s'", path, source_dir);
-        }
-        const char *path_part = path + source_dir_len;
-        while (*path_part == CHY_DIR_SEP_CHAR) {
-            ++path_part;
-        }
-
-        // Ignore hidden files.
-        if (path_part[0] == '.'
-            || strstr(path_part, CHY_DIR_SEP ".") != NULL) {
-            continue;
-        }
-
+        char *path_part = S_extract_path_part(path, source_dir, ".cfp");
         CFCFileSpec *file_spec
             = CFCFileSpec_new(source_dir, path_part, is_included);
         CFCParcel *parcel = CFCParcel_new_from_file(path, file_spec);
@@ -305,9 +285,10 @@ S_parse_parcel_files(const char *source_dir, int is_included) {
         }
         CFCBase_decref((CFCBase*)parcel);
         CFCBase_decref((CFCBase*)file_spec);
+        FREEMEM(path_part);
     }
 
-    S_free_find_files_context(&context);
+    CFCUtil_free_string_array(context.paths);
 }
 
 static void
@@ -340,33 +321,12 @@ S_parse_cf_files(CFCHierarchy *self, const char *source_dir, int is_included) {
     context.paths     = (char**)CALLOCATE(1, sizeof(char*));
     context.num_paths = 0;
     CFCUtil_walk(source_dir, S_find_files, &context);
-    size_t source_dir_len  = strlen(source_dir);
-    char *path_part = NULL;
-    size_t path_part_max = 0;
 
     // Process any file that has at least one class declaration.
     for (int i = 0; context.paths[i] != NULL; i++) {
         // Derive the name of the class that owns the module file.
         char *source_path = context.paths[i];
-        size_t source_path_len = strlen(source_path);
-        if (strncmp(source_path, source_dir, source_dir_len) != 0) {
-            CFCUtil_die("'%s' doesn't start with '%s'", source_path,
-                        source_dir);
-        }
-        size_t path_part_len = source_path_len
-                               - source_dir_len
-                               - strlen(".cfh");
-        if (path_part_max < path_part_len + 1) {
-            path_part_max = path_part_len + 1;
-            path_part = (char*)REALLOCATE(path_part, path_part_max);
-        }
-        const char *src = source_path + source_dir_len;
-        while (*src == CHY_DIR_SEP_CHAR) {
-            ++src;
-            --path_part_len;
-        }
-        memcpy(path_part, src, path_part_len);
-        path_part[path_part_len] = '\0';
+        char *path_part = S_extract_path_part(source_path, source_dir, ".cfh");
 
         // Ignore hidden files.
         if (path_part[0] == '.'
@@ -409,11 +369,31 @@ S_parse_cf_files(CFCHierarchy *self, const char *source_dir, int is_included) {
 
         CFCBase_decref((CFCBase*)file);
         CFCBase_decref((CFCBase*)file_spec);
+        FREEMEM(path_part);
     }
     self->classes[self->num_classes] = NULL;
 
-    S_free_find_files_context(&context);
-    FREEMEM(path_part);
+    CFCUtil_free_string_array(context.paths);
+}
+
+static void
+S_find_doc_files(const char *source_dir) {
+    CFCFindFilesContext context;
+    context.ext       = ".md";
+    context.paths     = (char**)CALLOCATE(1, sizeof(char*));
+    context.num_paths = 0;
+    CFCUtil_walk(source_dir, S_find_files, &context);
+
+    for (int i = 0; context.paths[i] != NULL; i++) {
+        char *path = context.paths[i];
+        char *path_part = S_extract_path_part(path, source_dir, ".md");
+        CFCDocument *doc = CFCDocument_create(path, path_part);
+
+        CFCBase_decref((CFCBase*)doc);
+        FREEMEM(path_part);
+    }
+
+    CFCUtil_free_string_array(context.paths);
 }
 
 static void
@@ -436,12 +416,34 @@ S_find_files(const char *path, void *arg) {
     }
 }
 
-static void
-S_free_find_files_context(CFCFindFilesContext *context) {
-    for (int i = 0; context->paths[i] != NULL; i++) {
-        FREEMEM(context->paths[i]);
+static char*
+S_extract_path_part(const char *path, const char *dir, const char *ext) {
+    size_t path_len = strlen(path);
+    size_t dir_len  = strlen(dir);
+    size_t ext_len  = strlen(ext);
+
+    if (path_len <= dir_len + ext_len) {
+        CFCUtil_die("Unexpected path '%s'", path);
     }
-    FREEMEM(context->paths);
+    if (strncmp(path, dir, dir_len) != 0) {
+        CFCUtil_die("'%s' doesn't start with '%s'", path, dir);
+    }
+    if (strcmp(path + path_len - ext_len, ext) != 0) {
+        CFCUtil_die("'%s' doesn't end with '%s'", path, ext);
+    }
+
+    const char *src = path + dir_len;
+    size_t path_part_len = path_len - (dir_len + ext_len);
+    while (path_part_len && *src == CHY_DIR_SEP_CHAR) {
+        ++src;
+        --path_part_len;
+    }
+
+    char *path_part = (char*)MALLOCATE(path_part_len + 1);
+    memcpy(path_part, src, path_part_len);
+    path_part[path_part_len] = '\0';
+
+    return path_part;
 }
 
 static void
@@ -457,7 +459,7 @@ S_connect_classes(CFCHierarchy *self) {
                     CFCUtil_die("Parent class '%s' not defined", parent_name);
                 }
                 const char *maybe_parent_name
-                    = CFCClass_get_class_name(maybe_parent);
+                    = CFCClass_get_name(maybe_parent);
                 if (strcmp(parent_name, maybe_parent_name) == 0) {
                     CFCClass_add_child(maybe_parent, klass);
                     break;
@@ -515,8 +517,8 @@ S_do_propagate_modified(CFCHierarchy *self, CFCClass *klass, int modified) {
         CFCClass *kid = children[i];
         if (CFCClass_final(klass)) {
             CFCUtil_die("Attempt to inherit from final class '%s' by '%s'",
-                        CFCClass_get_class_name(klass),
-                        CFCClass_get_class_name(kid));
+                        CFCClass_get_name(klass),
+                        CFCClass_get_name(kid));
         }
         if (S_do_propagate_modified(self, kid, modified)) {
             somebody_is_modified = 1;
@@ -601,10 +603,10 @@ S_add_file(CFCHierarchy *self, CFCFile *file) {
         CFCFile *existing = self->files[i];
         CFCClass **existing_classes = CFCFile_classes(existing);
         for (size_t j = 0; classes[j] != NULL; j++) {
-            const char *new_class_name = CFCClass_get_class_name(classes[j]);
+            const char *new_class_name = CFCClass_get_name(classes[j]);
             for (size_t k = 0; existing_classes[k] != NULL; k++) {
                 const char *existing_class_name
-                    = CFCClass_get_class_name(existing_classes[k]);
+                    = CFCClass_get_name(existing_classes[k]);
                 if (strcmp(new_class_name, existing_class_name) == 0) {
                     CFCUtil_die("Class '%s' already registered",
                                 new_class_name);
