@@ -23,6 +23,7 @@
 #include "CFCPerlClass.h"
 #include "CFCUtil.h"
 #include "CFCClass.h"
+#include "CFCFunction.h"
 #include "CFCMethod.h"
 #include "CFCParcel.h"
 #include "CFCParamList.h"
@@ -32,6 +33,7 @@
 #include "CFCVariable.h"
 #include "CFCType.h"
 #include "CFCPerlPod.h"
+#include "CFCPerlSub.h"
 #include "CFCPerlMethod.h"
 #include "CFCPerlConstructor.h"
 #include "CFCPerlTypeMap.h"
@@ -101,10 +103,7 @@ CFCPerlClass_destroy(CFCPerlClass *self) {
     }
     FREEMEM(self->cons_aliases);
     FREEMEM(self->cons_inits);
-    for (size_t i = 0; i < self->num_class_aliases; i++) {
-        FREEMEM(self->class_aliases[i]);
-    }
-    FREEMEM(self->class_aliases);
+    CFCUtil_free_string_array(self->class_aliases);
     CFCBase_destroy((CFCBase*)self);
 }
 
@@ -179,7 +178,7 @@ CFCPerlClass_bind_method(CFCPerlClass *self, const char *alias,
         CFCUtil_die("Can't bind_method %s -- can't find method %s in %s",
                     alias, meth_name, self->class_name);
     }
-    if (strcmp(CFCMethod_get_class_name(method), self->class_name) != 0) {
+    if (!CFCMethod_is_fresh(method, self->client)) {
         CFCUtil_die("Can't bind_method %s -- method %s not fresh in %s",
                     alias, meth_name, self->class_name);
     }
@@ -197,7 +196,7 @@ CFCPerlClass_exclude_method(CFCPerlClass *self, const char *meth_name) {
         CFCUtil_die("Can't exclude_method %s -- method not found in %s",
                     meth_name, self->class_name);
     }
-    if (strcmp(CFCMethod_get_class_name(method), self->class_name) != 0) {
+    if (!CFCMethod_is_fresh(method, self->client)) {
         CFCUtil_die("Can't exclude_method %s -- method not fresh in %s",
                     meth_name, self->class_name);
     }
@@ -226,29 +225,8 @@ CFCPerlClass_exclude_constructor(CFCPerlClass *self) {
     self->exclude_cons = 1;
 }
 
-static int
-S_can_be_bound(CFCParamList *param_list, CFCType *return_type) {
-    int success = 1;
-    CFCVariable **arg_vars = CFCParamList_get_variables(param_list);
-
-    for (size_t i = 0; arg_vars[i] != NULL; i++) {
-        CFCType *type = CFCVariable_get_type(arg_vars[i]);
-        char *conversion = CFCPerlTypeMap_from_perl(type, "foo");
-        if (conversion) { FREEMEM(conversion); }
-        else            { success = 0; }
-    }
-    if (!CFCType_is_void(return_type)) {
-        char *conversion = CFCPerlTypeMap_to_perl(return_type, "foo");
-        if (conversion) { FREEMEM(conversion); }
-        else            { success = 0; }
-    }
-
-    return success;
-}
-
 CFCPerlMethod**
 CFCPerlClass_method_bindings(CFCClass *klass) {
-    CFCClass       *parent        = CFCClass_get_parent(klass);
     size_t          num_bound     = 0;
     CFCMethod     **fresh_methods = CFCClass_fresh_methods(klass);
     CFCPerlMethod **bound 
@@ -258,18 +236,13 @@ CFCPerlClass_method_bindings(CFCClass *klass) {
     for (size_t i = 0; fresh_methods[i] != NULL; i++) {
         CFCMethod *method = fresh_methods[i];
 
-        // Skip private methods.
-        if (CFCSymbol_private((CFCSymbol*)method)) { continue; }
-
         // Skip methods which have been explicitly excluded.
         if (CFCMethod_excluded_from_host(method)) {
             continue;
         }
 
-        // Skip methods with types which cannot be mapped automatically.
-        CFCParamList *param_list  = CFCMethod_get_param_list(method);
-        CFCType      *return_type = CFCMethod_get_return_type(method);
-        if (!S_can_be_bound(param_list, return_type)) {
+        // Skip methods that shouldn't be bound.
+        if (!CFCMethod_can_be_bound(method)) {
             continue;
         }
 
@@ -281,7 +254,7 @@ CFCPerlClass_method_bindings(CFCClass *klass) {
          * this way allows SUPER:: invocations from Perl-space to work
          * properly.
          */
-        CFCPerlMethod *meth_binding = CFCPerlMethod_new(method);
+        CFCPerlMethod *meth_binding = CFCPerlMethod_new(klass, method);
         size_t size = (num_bound + 2) * sizeof(CFCPerlMethod*);
         bound = (CFCPerlMethod**)REALLOCATE(bound, size);
         bound[num_bound] = meth_binding;
@@ -289,8 +262,6 @@ CFCPerlClass_method_bindings(CFCClass *klass) {
         bound[num_bound] = NULL;
     }
 
-    FREEMEM(fresh_methods);
-
     return bound;
 }
 
@@ -298,7 +269,7 @@ static const char NEW[] = "new";
 
 CFCPerlConstructor**
 CFCPerlClass_constructor_bindings(CFCClass *klass) {
-    const char    *class_name = CFCClass_get_class_name(klass);
+    const char    *class_name = CFCClass_get_name(klass);
     CFCPerlClass  *perl_class = CFCPerlClass_singleton(class_name);
     CFCFunction  **functions  = CFCClass_functions(klass);
     size_t         num_bound  = 0;
@@ -307,29 +278,27 @@ CFCPerlClass_constructor_bindings(CFCClass *klass) {
 
     // Iterate over the list of possible initialization functions.
     for (size_t i = 0; functions[i] != NULL; i++) {
-        CFCFunction  *function    = functions[i];
-        const char   *micro_sym   = CFCFunction_micro_sym(function);
-        CFCParamList *param_list  = CFCFunction_get_param_list(function);
-        CFCType      *return_type = CFCFunction_get_return_type(function);
-        const char   *alias       = NULL;
+        CFCFunction  *function  = functions[i];
+        const char   *func_name = CFCFunction_get_name(function);
+        const char   *alias     = NULL;
 
         // Find user-specified alias.
         if (perl_class == NULL) {
             // Bind init() to new() when possible.
-            if (strcmp(micro_sym, "init") == 0
-                && S_can_be_bound(param_list, return_type)
+            if (strcmp(func_name, "init") == 0
+                && CFCFunction_can_be_bound(function)
                ) {
                 alias = NEW;
             }
         }
         else {
             for (size_t j = 0; j < perl_class->num_cons; j++) {
-                if (strcmp(micro_sym, perl_class->cons_inits[j]) == 0) {
+                if (strcmp(func_name, perl_class->cons_inits[j]) == 0) {
                     alias = perl_class->cons_aliases[j];
-                    if (!S_can_be_bound(param_list, return_type)) {
+                    if (!CFCFunction_can_be_bound(function)) {
                         CFCUtil_die("Can't bind %s as %s"
                                     " -- types can't be mapped",
-                                    micro_sym, alias);
+                                    func_name, alias);
                     }
                     break;
                 }
@@ -338,8 +307,8 @@ CFCPerlClass_constructor_bindings(CFCClass *klass) {
             // Automatically bind init() to new() when possible.
             if (!alias
                 && !perl_class->exclude_cons
-                && strcmp(micro_sym, "init") == 0
-                && S_can_be_bound(param_list, return_type)
+                && strcmp(func_name, "init") == 0
+                && CFCFunction_can_be_bound(function)
                ) {
                 int saw_new = 0;
                 for (size_t j = 0; j < perl_class->num_cons; j++) {
@@ -359,7 +328,7 @@ CFCPerlClass_constructor_bindings(CFCClass *klass) {
 
         // Create the binding, add it to the array.
         CFCPerlConstructor *cons_binding
-            = CFCPerlConstructor_new(klass, alias, micro_sym);
+            = CFCPerlConstructor_new(klass, alias, func_name);
         size_t size = (num_bound + 2) * sizeof(CFCPerlConstructor*);
         bound = (CFCPerlConstructor**)REALLOCATE(bound, size);
         bound[num_bound] = cons_binding;
@@ -388,14 +357,18 @@ CFCPerlClass_create_pod(CFCPerlClass *self) {
 
     // Get the class's brief description.
     const char *raw_brief = CFCDocuComment_get_brief(docucom);
-    char *brief = CFCPerlPod_perlify_doc_text(pod_spec, raw_brief);
+    char *brief = CFCPerlPod_md_to_pod(raw_brief, client, 2);
 
     // Get the class's long description.
-    const char *raw_description = CFCPerlPod_get_description(pod_spec);
-    if (!raw_description || !strlen(raw_description)) {
-        raw_description = CFCDocuComment_get_long(docucom);
+    char *description;
+    const char *pod_description = CFCPerlPod_get_description(pod_spec);
+    if (pod_description && strlen(pod_description)) {
+        description = CFCUtil_sprintf("%s\n", pod_description);
+    }
+    else {
+        const char *raw_description = CFCDocuComment_get_long(docucom);
+        description = CFCPerlPod_md_to_pod(raw_description, client, 2);
     }
-    char *description = CFCPerlPod_perlify_doc_text(pod_spec, raw_description);
 
     // Create SYNOPSIS.
     const char *raw_synopsis = CFCPerlPod_get_synopsis(pod_spec);
@@ -418,7 +391,7 @@ CFCPerlClass_create_pod(CFCPerlClass *self) {
                                   class_name, NULL);
         CFCClass *ancestor = client;
         while (NULL != (ancestor = CFCClass_get_parent(ancestor))) {
-            const char *ancestor_klass = CFCClass_get_class_name(ancestor);
+            const char *ancestor_klass = CFCClass_get_name(ancestor);
             if (CFCPerlClass_singleton(ancestor_klass)) {
                 inheritance = CFCUtil_cat(inheritance, " isa L<",
                                           ancestor_klass, ">", NULL);
@@ -428,46 +401,25 @@ CFCPerlClass_create_pod(CFCPerlClass *self) {
                                           ancestor_klass, NULL);
             }
         }
-        inheritance = CFCUtil_cat(inheritance, ".\n", NULL);
+        inheritance = CFCUtil_cat(inheritance, ".\n\n", NULL);
     }
 
     // Put it all together.
-    const char pattern[] = 
-    "# Auto-generated file -- DO NOT EDIT!!!!!\n"
-    "\n"
-    "# Licensed to the Apache Software Foundation (ASF) under one or more\n"
-    "# contributor license agreements.  See the NOTICE file distributed with\n"
-    "# this work for additional information regarding copyright ownership.\n"
-    "# The ASF licenses this file to You under the Apache License, Version 2.0\n"
-    "# (the \"License\"); you may not use this file except in compliance with\n"
-    "# the License.  You may obtain a copy of the License at\n"
-    "#\n"
-    "#     http://www.apache.org/licenses/LICENSE-2.0\n"
-    "#\n"
-    "# Unless required by applicable law or agreed to in writing, software\n"
-    "# distributed under the License is distributed on an \"AS IS\" BASIS,\n"
-    "# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\n"
-    "# See the License for the specific language governing permissions and\n"
-    "# limitations under the License.\n"
-    "\n"
-    "=head1 NAME\n"
-    "\n"
-    "%s - %s\n"
-    "\n"
-    "%s\n"
-    "\n"
-    "=head1 DESCRIPTION\n"
-    "\n"
-    "%s\n"
-    "\n"
-    "%s\n"
-    "\n"
-    "%s\n"
-    "\n"
-    "%s\n"
-    "\n"
-    "=cut\n"
-    "\n";
+    const char pattern[] =
+        "=encoding utf8\n"
+        "\n"
+        "=head1 NAME\n"
+        "\n"
+        "%s - %s"
+        "%s"
+        "=head1 DESCRIPTION\n"
+        "\n"
+        "%s"
+        "%s"
+        "%s"
+        "%s"
+        "=cut\n"
+        "\n";
     char *pod
         = CFCUtil_sprintf(pattern, class_name, brief, synopsis, description,
                           constructor_pod, methods_pod, inheritance);
@@ -548,20 +500,19 @@ CFCPerlClass_method_metadata_code(CFCPerlClass *self) {
         CFCMethod *method = fresh_methods[i];
         if (!CFCMethod_novel(method)) { continue; }
 
-        const char *macro_sym = CFCMethod_get_macro_sym(method);
+        const char *meth_name = CFCMethod_get_name(method);
         const char *alias     = CFCMethod_get_host_alias(method);
         if (alias) {
             code = CFCUtil_cat(code, "    CFISH_Class_Add_Host_Method_Alias(",
-                               class_var, ", \"", alias, "\", \"", macro_sym,
+                               class_var, ", \"", alias, "\", \"", meth_name,
                                "\");\n", NULL);
         }
         if (CFCMethod_excluded_from_host(method)) {
             code = CFCUtil_cat(code, "    CFISH_Class_Exclude_Host_Method(",
-                               class_var, ", \"", macro_sym, "\");\n", NULL);
+                               class_var, ", \"", meth_name, "\");\n", NULL);
         }
     }
 
-    FREEMEM(fresh_methods);
     return code;
 }
 
