@@ -71,15 +71,15 @@ S_gen_code_sample(CFCCallable *func, const char *alias, CFCClass *klass,
                   int is_constructor);
 
 static char*
-S_gen_positional_sample(const char *invocant, const char *alias,
-                        CFCParamList *param_list, size_t start);
+S_gen_positional_sample(const char *prologue, CFCParamList *param_list,
+                        size_t start);
 
 static char*
-S_gen_labeled_sample(const char *invocant, const char *alias,
-                     CFCParamList *param_list, size_t start);
+S_gen_labeled_sample(const char *prologue, CFCParamList *param_list,
+                     size_t start);
 
 static char*
-S_perl_var_name(CFCVariable *var);
+S_perl_var_name(CFCType *type, int is_ctor_retval);
 
 static char*
 S_camel_to_lower(const char *camel);
@@ -87,6 +87,9 @@ S_camel_to_lower(const char *camel);
 static char*
 S_nodes_to_pod(cmark_node *node, CFCClass *klass, int header_level);
 
+static char*
+S_node_to_pod(cmark_node *node, CFCClass *klass, int header_level);
+
 static char*
 S_pod_escape(const char *content);
 
@@ -389,38 +392,59 @@ CFCPerlPod_gen_subroutine_pod(CFCCallable *func,
 static char*
 S_gen_code_sample(CFCCallable *func, const char *alias, CFCClass *klass,
                   int is_constructor) {
-    char *invocant = NULL;
+    char *prologue       = CFCUtil_sprintf("");
+    char *class_var_name = S_camel_to_lower(CFCClass_get_struct_sym(klass));
+
+    CFCType *ret_type = CFCCallable_get_return_type(func);
+    if (!CFCType_is_void(ret_type)) {
+        char *ret_name = S_perl_var_name(ret_type, is_constructor);
+
+        if (!is_constructor && strcmp(ret_name, class_var_name) == 0) {
+            // Return type equals `klass`. Use a generic variable name
+            // to avoid confusing code samples like
+            // `my $string = $string->trim`.
+            prologue = CFCUtil_cat(prologue, "my $result = ", NULL);
+        }
+        else {
+            prologue = CFCUtil_cat(prologue, "my $", ret_name, " = ", NULL);
+        }
+
+        FREEMEM(ret_name);
+    }
 
     if (is_constructor) {
-        invocant = CFCUtil_strdup(CFCClass_get_name(klass));
+        const char *invocant = CFCClass_get_name(klass);
+        prologue = CFCUtil_cat(prologue, invocant, NULL);
     }
     else {
-        char *lower = S_camel_to_lower(CFCClass_get_struct_sym(klass));
-        invocant = CFCUtil_sprintf("$%s", lower);
-        FREEMEM(lower);
+        prologue = CFCUtil_cat(prologue, "$", class_var_name, NULL);
     }
 
+    prologue = CFCUtil_cat(prologue, "->", alias, NULL);
+
     CFCParamList *param_list = CFCCallable_get_param_list(func);
     size_t        num_vars   = CFCParamList_num_vars(param_list);
     size_t        start      = is_constructor ? 0 : 1;
     char         *sample     = NULL;
 
     if (start == num_vars) {
-        sample = CFCUtil_sprintf("    %s->%s();\n", invocant, alias);
+        sample = CFCUtil_sprintf("    %s();\n", prologue);
     }
     else if (is_constructor || num_vars - start >= 2) {
-        sample = S_gen_labeled_sample(invocant, alias, param_list, start);
+        sample = S_gen_labeled_sample(prologue, param_list, start);
     }
     else {
-        sample = S_gen_positional_sample(invocant, alias, param_list, start);
+        sample = S_gen_positional_sample(prologue, param_list, start);
     }
 
+    FREEMEM(class_var_name);
+    FREEMEM(prologue);
     return sample;
 }
 
 static char*
-S_gen_positional_sample(const char *invocant, const char *alias,
-                        CFCParamList *param_list, size_t start) {
+S_gen_positional_sample(const char *prologue, CFCParamList *param_list,
+                        size_t start) {
     size_t        num_vars = CFCParamList_num_vars(param_list);
     CFCVariable **vars     = CFCParamList_get_variables(param_list);
     const char  **inits    = CFCParamList_get_initial_values(param_list);
@@ -431,14 +455,13 @@ S_gen_positional_sample(const char *invocant, const char *alias,
     }
 
     const char *name = CFCVariable_get_name(vars[start]);
-    char *sample
-        = CFCUtil_sprintf("    %s->%s($%s);\n", invocant, alias, name);
+    char *sample = CFCUtil_sprintf("    %s($%s);\n", prologue, name);
 
     const char *init = inits[start];
     if (init) {
         if (strcmp(init, "NULL") == 0) { init = "undef"; }
-        char *def_sample = CFCUtil_sprintf("    %s->%s();  # default: %s\n",
-                                           invocant, alias, init);
+        char *def_sample = CFCUtil_sprintf("    %s();  # default: %s\n",
+                                           prologue, init);
         sample = CFCUtil_cat(sample, def_sample, NULL);
         FREEMEM(def_sample);
     }
@@ -447,39 +470,27 @@ S_gen_positional_sample(const char *invocant, const char *alias,
 }
 
 static char*
-S_gen_labeled_sample(const char *invocant, const char *alias,
-                     CFCParamList *param_list, size_t start) {
+S_gen_labeled_sample(const char *prologue, CFCParamList *param_list,
+                     size_t start) {
     size_t        num_vars = CFCParamList_num_vars(param_list);
     CFCVariable **vars     = CFCParamList_get_variables(param_list);
     const char  **inits    = CFCParamList_get_initial_values(param_list);
 
-    size_t max_label_len = 0;
-    size_t max_var_len   = 0;
+    size_t max_name_len = 0;
 
-    // Find maximum length of label and Perl variable.
+    // Find maximum length of parameter name.
     for (size_t i = start; i < num_vars; i++) {
-        CFCVariable *var = vars[i];
-
-        const char *label = CFCVariable_get_name(var);
-        size_t label_len = strlen(label);
-        if (label_len > max_label_len) { max_label_len = label_len; }
-
-        char *perl_var = S_perl_var_name(var);
-        size_t perl_var_len = strlen(perl_var);
-        if (perl_var_len > max_var_len) { max_var_len = perl_var_len; }
-        FREEMEM(perl_var);
+        const char *name = CFCVariable_get_name(vars[i]);
+        size_t name_len = strlen(name);
+        if (name_len > max_name_len) { max_name_len = name_len; }
     }
 
     char *params = CFCUtil_strdup("");
 
     for (size_t i = start; i < num_vars; i++) {
-        CFCVariable *var      = vars[i];
-        const char  *label    = CFCVariable_get_name(var);
-        char        *perl_var = S_perl_var_name(var);
-        perl_var = CFCUtil_cat(perl_var, ",", NULL);
-
-        char       *comment = NULL;
+        const char *name    = CFCVariable_get_name(vars[i]);
         const char *init    = inits[i];
+        char       *comment = NULL;
 
         if (init) {
             if (strcmp(init, "NULL") == 0) { init = "undef"; }
@@ -490,43 +501,49 @@ S_gen_labeled_sample(const char *invocant, const char *alias,
         }
 
         char *line = CFCUtil_sprintf("        %-*s => $%-*s  # %s\n",
-                                     max_label_len, label,
-                                     max_var_len + 1, perl_var,
+                                     (int)max_name_len, name,
+                                     (int)max_name_len, name,
                                      comment);
         params = CFCUtil_cat(params, line, NULL);
         FREEMEM(line);
         FREEMEM(comment);
-        FREEMEM(perl_var);
     }
 
     const char pattern[] =
-        "    %s->%s(\n"
+        "    %s(\n"
         "%s"
         "    );\n";
-    char *sample = CFCUtil_sprintf(pattern, invocant, alias, params);
+    char *sample = CFCUtil_sprintf(pattern, prologue, params);
 
     FREEMEM(params);
     return sample;
 }
 
 static char*
-S_perl_var_name(CFCVariable *var) {
-    CFCType    *type      = CFCVariable_get_type(var);
+S_perl_var_name(CFCType *type, int is_ctor_retval) {
     const char *specifier = CFCType_get_specifier(type);
     char       *perl_name = NULL;
 
     if (CFCType_is_object(type)) {
-        // Skip parcel prefix.
-        if (islower(*specifier)) {
-            for (specifier++; *specifier; specifier++) {
-                if (*specifier == '_') {
-                    specifier++;
-                    break;
+        if (!is_ctor_retval && strcmp(specifier, "cfish_Vector") == 0) {
+            perl_name = CFCUtil_strdup("arrayref");
+        }
+        else if (!is_ctor_retval && strcmp(specifier, "cfish_Hash") == 0) {
+            perl_name = CFCUtil_strdup("hashref");
+        }
+        else {
+            // Skip parcel prefix.
+            if (islower(*specifier)) {
+                for (specifier++; *specifier; specifier++) {
+                    if (*specifier == '_') {
+                        specifier++;
+                        break;
+                    }
                 }
             }
-        }
 
-        perl_name = S_camel_to_lower(specifier);
+            perl_name = S_camel_to_lower(specifier);
+        }
     }
     else if (CFCType_is_integer(type)) {
         if (strcmp(specifier, "bool") == 0) {
@@ -575,21 +592,78 @@ S_camel_to_lower(const char *camel) {
 }
 
 char*
-CFCPerlPod_md_to_pod(const char *md, CFCClass *klass, int header_level) {
-    int options = CMARK_OPT_NORMALIZE
-                  | CMARK_OPT_SMART
-                  | CMARK_OPT_VALIDATE_UTF8
-                  | CMARK_OPT_SAFE;
+CFCPerlPod_md_doc_to_pod(const char *module, const char *md) {
+    int options = CMARK_OPT_SMART
+                  | CMARK_OPT_VALIDATE_UTF8;
     cmark_node *doc = cmark_parse_document(md, strlen(md), options);
-    char *pod = S_nodes_to_pod(doc, klass, header_level);
+    cmark_node *maybe_header = cmark_node_first_child(doc);
+    char *name;
+    char *desc;
+
+    if (maybe_header
+        && cmark_node_get_type(maybe_header) == CMARK_NODE_HEADER
+       ) {
+        cmark_node *header_child = cmark_node_first_child(maybe_header);
+        char *short_desc = S_nodes_to_pod(header_child, NULL, 1);
+        name = CFCUtil_sprintf("%s - %s", module, short_desc);
+        FREEMEM(short_desc);
+
+        cmark_node *remaining = cmark_node_next(maybe_header);
+        desc = S_nodes_to_pod(remaining, NULL, 1);
+    }
+    else {
+        // No header found.
+        name = CFCUtil_strdup(module);
+        desc = S_node_to_pod(doc, NULL, 1);
+    }
+
+    const char *pattern =
+        "=head1 NAME\n"
+        "\n"
+        "%s\n"
+        "\n"
+        "=head1 DESCRIPTION\n"
+        "\n"
+        "%s";
+    char *retval = CFCUtil_sprintf(pattern, name, desc);
+
+    FREEMEM(name);
+    FREEMEM(desc);
+    cmark_node_free(doc);
+    return retval;
+}
+
+char*
+CFCPerlPod_md_to_pod(const char *md, CFCClass *klass, int header_level) {
+    int options = CMARK_OPT_SMART
+                  | CMARK_OPT_VALIDATE_UTF8;
+    cmark_node *doc = cmark_parse_document(md, strlen(md), options);
+    char *pod = S_node_to_pod(doc, klass, header_level);
     cmark_node_free(doc);
 
     return pod;
 }
 
+// Convert a node and its siblings.
 static char*
 S_nodes_to_pod(cmark_node *node, CFCClass *klass, int header_level) {
     char *result = CFCUtil_strdup("");
+
+    while (node != NULL) {
+        char *pod = S_node_to_pod(node, klass, header_level);
+        result = CFCUtil_cat(result, pod, NULL);
+        FREEMEM(pod);
+
+        node = cmark_node_next(node);
+    }
+
+    return result;
+}
+
+// Convert a single node.
+static char*
+S_node_to_pod(cmark_node *node, CFCClass *klass, int header_level) {
+    char *result = CFCUtil_strdup("");
     if (node == NULL) {
         return result;
     }
