@@ -244,7 +244,7 @@ CFCTest_test_string_equals(CFCTest *self, const char *result,
 }
 
 void
-CFCTest_test_int_equals(CFCTest *self, long result, long expected,
+CFCTest_test_int_equals(CFCTest *self, uint64_t result, uint64_t expected,
                         const char *fmt, ...) {
     int cond = (result == expected);
 
@@ -254,7 +254,8 @@ CFCTest_test_int_equals(CFCTest *self, long result, long expected,
     va_end(args);
 
     if (!cond) {
-        self->formatter->test_comment("Expected '%ld', got '%ld'.\n",
+        self->formatter->test_comment("Expected '%"PRIu64"',"
+                                      " got '%"PRIu64"'.\n",
                                       expected, result);
     }
 }
