@@ -17,6 +17,7 @@
 #define CFC_USE_TEST_MACROS
 #include "CFCBase.h"
 #include "CFCDocuComment.h"
+#include "CFCDocument.h"
 #include "CFCCHtml.h"
 #include "CFCCMan.h"
 #include "CFCClass.h"
@@ -38,7 +39,7 @@ S_run_tests(CFCTest *test);
 
 const CFCTestBatch CFCTEST_BATCH_DOCU_COMMENT = {
     "Clownfish::CFC::Model::DocuComment",
-    18,
+    19,
     S_run_tests
 };
 
@@ -117,6 +118,19 @@ S_test_parser(CFCTest *test) {
     CFCBase_decref((CFCBase*)parser);
 }
 
+static void
+S_test_md_to_pod(CFCTest *test) {
+    const char *md =
+        "[Link\n"
+        "with newline](http://example.com/)\n";
+    char *pod = CFCPerlPod_md_to_pod(md, NULL, 1);
+    const char *expect =
+        "L<Link\n"
+        "with newline|http://example.com/>\n\n";
+    STR_EQ(test, pod, expect, "Markdown link with newline to POD");
+    FREEMEM(pod);
+}
+
 static void
 S_test_generator(CFCTest *test) {
     CFCHierarchy *hierarchy = CFCHierarchy_new("autogen");
@@ -300,12 +314,14 @@ S_test_generator(CFCTest *test) {
     CFCBase_decref((CFCBase*)parcel);
     CFCBase_decref((CFCBase*)hierarchy);
 
+    CFCDocument_clear_registry();
     CFCParcel_reap_singletons();
 }
 
 static void
 S_run_tests(CFCTest *test) {
     S_test_parser(test);
+    S_test_md_to_pod(test);
     S_test_generator(test);
 }
 
