@@ -99,7 +99,7 @@ S_run_basic_tests(CFCTest *test) {
     TEST_BOOL_ACCESSOR(type, is_primitive);
     TEST_BOOL_ACCESSOR(type, is_integer);
     TEST_BOOL_ACCESSOR(type, is_floating);
-    TEST_BOOL_ACCESSOR(type, is_string_type);
+    TEST_BOOL_ACCESSOR(type, cfish_string);
     TEST_BOOL_ACCESSOR(type, is_va_list);
     TEST_BOOL_ACCESSOR(type, is_arbitrary);
     TEST_BOOL_ACCESSOR(type, is_composite);
@@ -331,7 +331,7 @@ S_run_object_tests(CFCTest *test) {
     CFCParcel *neato_parcel = CFCParcel_new("Neato", NULL, NULL, NULL);
     CFCClass *foo_class
         = CFCClass_create(neato_parcel, NULL, "Foo", NULL, NULL, NULL, NULL,
-                          NULL, false, false);
+                          false, false, false);
     CFCType *foo = CFCType_new_object(0, neato_parcel, "Foo", 1);
     CFCType_resolve(foo);
 
@@ -345,7 +345,7 @@ S_run_object_tests(CFCTest *test) {
     {
         CFCClass *bar_class
             = CFCClass_create(neato_parcel, NULL, "Bar", NULL, NULL, NULL,
-                              NULL, NULL, false, false);
+                              NULL, false, false, false);
         CFCType *bar = CFCType_new_object(0, neato_parcel, "Bar", 1);
         CFCType_resolve(bar);
         OK(test, !CFCType_equals(foo, bar),
@@ -359,7 +359,7 @@ S_run_object_tests(CFCTest *test) {
             = CFCParcel_new("Foreign", NULL, NULL, NULL);
         CFCClass *foreign_foo_class
             = CFCClass_create(foreign_parcel, NULL, "Foreign::Foo", NULL, NULL,
-                              NULL, NULL, NULL, false, false);
+                              NULL, NULL, false, false, false);
         CFCType *foreign_foo = CFCType_new_object(0, foreign_parcel, "Foo", 1);
         CFCType_resolve(foreign_foo);
         OK(test, !CFCType_equals(foo, foreign_foo),
@@ -389,8 +389,8 @@ S_run_object_tests(CFCTest *test) {
     {
         CFCType *string_type
             = CFCType_new_object(0, neato_parcel, "String", 1);
-        OK(test, CFCType_is_string_type(string_type), "%s", "is_string_type");
-        OK(test, !CFCType_is_string_type(foo), "not %s", "not is_string_type");
+        OK(test, CFCType_cfish_string(string_type), "%s", "cfish_string");
+        OK(test, !CFCType_cfish_string(foo), "not %s", "not cfish_string");
         CFCBase_decref((CFCBase*)string_type);
     }
 
